# File tres/tharset.cl
# March 6, 2009
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# THARSET -- Set a specific savelength-dispersed ThAr spectrum
#	     to be a local standard for its configuration

procedure tharset (spectrum)

file	spectrum=""	{prompt="ThAr file with dispersion"}

string	comproot="comp"	{prompt="ThAr file root name"}
string	comppath=""	{prompt="ThAr date and file name (returned)"}
bool	verbose=yes	{prompt="Print confirmation of file setting (yes or no)"}

begin

    file cfile, cdb, sdb, spec
    string croot, mjdmid
    real shift
    bool vb

    vb = verbose

# Prepend date directory to filename
    gettime (tfib1.mjd)
    comppath = gettime.datadir // "/" // spectrum

# Make reference file name from root and apfib
    spec = spectrum
    tfib1 (spec)
    croot = comproot // tfib1.apfib

# Copy this thorium argon spectrum file to the reference file name
    cfile = croot // ".ec.fits"
    if (access (cfile))
	delete (cfile)
    copy (spec, cfile)

# Set COMPFILE in reference file to name of source ThAr file
    hedit (cfile, "COMPFILE", comppath,add+,update+,verify-,show=vb)

# Set shift in this file to zero relative to itself
    shift = 0.0
    hedit (spec, "TIDSHIFT", shift, add+, update+,verify-,show=vb)
    hedit (spec, "TIDREF", comppath, add+, update+,verify-,show=vb)
    round (tfib1.mjdmid, "%12.6f")
    mjdmid = round.value
    hedit (spec, "TIDMJD", tfib1.mjdmid, add+, update+,verify-,show=vb)

# Copy thorium argon spectrum database/id file to standard name
    tmodid (spec, cfile, indir="", outdir="", shift=0.0, keepfit+, verbose=vb)
    if (verbose)
	print ("THARSET: ",spec," -> ", cfile)
 
end

# Aug  8 2008	New task
# Oct  1 2008	Write COMPFILE keyword to output file header to give source
# Oct  7 2008	Set vb from verbose
# Dec 18 2008	Add TIDSHIFT=0, TIDREF, and TIDMJD to FITS header
# Dec 18 2008	Return comppath for use in table
# Dec 31 2008	Add more comments

# Mar  6 2009	Use TMODID to set names in database file correctly
