# File tres/Scripts/trsproc.cl
# March 10, 2009
# By Doug Mink

# TRSPROC: Process TRES data from lists of files made by TRSGROUP

procedure trsproc (sproc)

string	sproc="bdfto"	{prompt="Process b=ias d=ark f=lat c/t=ThAr o=bject spectra"}

string	images="1-300"	{prompt="Sequence number range for images"}
string	datedir=""	{prompt="Date for images"}
file	instdir=""	{prompt="Directory containing date directories"}
file	imagedir=""	{prompt="Directory containing images (overrides above)"}
file	datafile="data.db"	{prompt="File for observation database"}
file	biaslist="bias.list"	{prompt="File listing groups of bias images"}
file	darklist="dark.list"	{prompt="File listing groups of dark images"}
file	flatlist="flat.list"	{prompt="File listing groups of flat fields"}
file	complist="comp.list"	{prompt="File listing groups of ThAr exposures"}
file	objlist="obj.list"	{prompt="File listing groups of observations"}
file	logfile="trsproc.log"	{prompt="Log file name (stdout if null string)"}
bool	saveraw=no	{prompt="Save unprocessed image files (y or n)"}
bool	verbose=yes	{prompt="Summarize results (yes or no)"}
bool	debug=no	{prompt="Print file sorting instead of writing files (yes or no)"}

struct *proclist

begin
    file listfile, lf
    string filelist, ims
    int ngroups, nbias, ndark, nobj, ncomp, nflat
    bool sraw

    ims = images
    sraw = saveraw
    lf = logfile
    if (lf == "")
	lf = stdout

# Initialize TRSGROUP
    trsgroup.datedir = datedir
    trsgroup.instdir = instdir
    trsgroup.imagedir = imagedir
    trsgroup.datafile = datafile
    trsgroup.verbose = verbose
    trsgroup.debug = debug

    nflat = 0
    nbias = 0
    ndark = 0
    ncomp = 0
    nobj = 0

    tcheck
    if (tcheck.rawdir)
	trsdate (tcheck.datedir)

# Process flat field files from lists
    smatch (sproc, "f")
    if (smatch.found) {
	if (!access (flatlist))
	    trsgroup (ims)
	proclist = flatlist
    	while (fscan (proclist,listfile) != EOF) {
	    if (substr (listfile,1,1) == "#") {
		print ("TRSPROC: Skipping file ",listfile," from ", flatlist, >>lf)
		}
	    else {
		print ("TRSPROC: Processing files ",listfile," from ",flatlist, >>lf)
		filelist = "@" // listfile
		ftres (filelist, saveraw=sraw)
		nflat = nflat + 1
		}
	    }
	}

# Process bias files from list
    smatch (sproc, "b")
    if (smatch.found) {
	if (!access (biaslist))
	    trsgroup (ims)
	proclist = biaslist
	while (fscan (proclist,listfile) != EOF) {
	    if (substr(listfile,1,1) == "#") {
		print ("TRSPROC: Skipping file ",listfile," from ", biaslist, >>lf)
		}
	    else {
		print ("TRSPROC: Processing files ",listfile," from ", biaslist, >>lf)
		filelist = "@" // listfile
		ftres (filelist, saveraw=sraw)
		nbias = nbias + 1
		}
	    }
	}

# Process dark files from lists
    smatch (sproc, "d")
    if (smatch.found) {
	if (!access (darklist))
	    trsgroup (ims)
	proclist = darklist
	while (fscan (proclist,listfile) != EOF) {
	    if (substr(listfile,1,1) == "#") {
		print ("TRSPROC: Skipping file ",listfile," from ", darklist, >>lf)
		}
	    else {
		print ("TRSPROC: Processing files ",listfile," from ", darklist, >>lf)
		filelist = "@" // listfile
		ftres (filelist, saveraw=sraw)
		ndark = ndark + 1
		}
	    }
	}

# Process Thorium Argon comparison lamp files from lists
    smatch (sproc, "t")
    if (!smatch.found)
	smatch (sproc, "c")
    if (smatch.found) {
	if (!access (complist))
	    trsgroup (ims)
	proclist = complist
	while (fscan (proclist,listfile) != EOF) {
	    if (substr(listfile,1,1) == "#") {
		print ("TRSPROC: Skipping file ",listfile," from ", complist, >>lf)
		}
	    else {
		print ("TRSPROC: Processing files ",listfile," from ", complist, >>lf)
		filelist = "@" // listfile
		ttres (filelist, saveraw=sraw)
		ncomp = ncomp + 1
		}
	    }
	}

# Process object and sky files from lists
    smatch (sproc, "o")
    if (smatch.found) {
	if (!access (objlist))
	    trsgroup (ims)
	proclist = objlist
	while (fscan (proclist,listfile) != EOF) {
	    if (substr(listfile,1,1) == "#") {
		print ("TRSPROC: Skipping file ",listfile," from ", objlist, >>lf)
		}
	    else {
		print ("TRSPROC: Processing files ",listfile," from ", objlist, >>lf)
		filelist = "@" // listfile
		ctres (filelist, saveraw=sraw)
		nobj = nobj + 1
		}
	    }
	}
    
# Summarize the sorting process
    ngroups = nobj + nbias + ndark + nflat + ncomp
    if (verbose)
	print ("TRSPROC: ", ngroups, " groups of spectra processed", >>lf)

    if (tcheck.rawdir)
	tdata (datedir=tcheck.datedir)

    proclist = ""
end

# Jul 31 2008	New task
# Aug 14 2008	Use file names from trsgroup; set type(s) to process from stype
# Aug 15 2008	Move filenames back from trsgroup; separate dark and bias
# Sep  9 2008	Fix declarations
# Sep 19 2008	Add images parameter to set range of sequence numbers to check
# Oct 14 2008	Do not run if in raw data directory
# Oct 14 2008	Pass defaulted-to-no saveraw parameter to *tres tasks
# Oct 15 2008	If run from raw data directory, switch to quick-look directory
# Oct 22 2008	Skip list files which are commented out with a #
# Nov 17 2008	Drop parameter sortfile
# Dec 31 2008	Add logfile parameter

# Mar 10 2009	Skip flat files which are commented out with a #
